package com.ruoyi.postCheck.mapper;

import java.util.List;
import com.ruoyi.postCheck.domain.PostProgramme;
import com.baomidou.mybatisplus.core.mapper.BaseMapper;

/**
 * 岗检方案Mapper接口
 *
 * @author csg
 * @date 2023-12-19
 */
public interface PostProgrammeMapper extends BaseMapper<PostProgramme>
{
    /**
     * 查询岗检方案
     *
     * @param id 岗检方案主键
     * @return 岗检方案
     */
    public PostProgramme selectPostProgrammeById(Long id);

    /**
     * 查询岗检方案列表
     *
     * @param postProgramme 岗检方案
     * @return 岗检方案集合
     */
    public List<PostProgramme> selectPostProgrammeList(PostProgramme postProgramme);

    /**
     * 新增岗检方案
     *
     * @param postProgramme 岗检方案
     * @return 结果
     */
    public int insertPostProgramme(PostProgramme postProgramme);

    /**
     * 修改岗检方案
     *
     * @param postProgramme 岗检方案
     * @return 结果
     */
    public int updatePostProgramme(PostProgramme postProgramme);

    /**
     * 删除岗检方案
     *
     * @param id 岗检方案主键
     * @return 结果
     */
    public int deletePostProgrammeById(Long id);

    /**
     * 批量删除岗检方案
     *
     * @param ids 需要删除的数据主键集合
     * @return 结果
     */
    public int deletePostProgrammeByIds(Long[] ids);
}
